/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap;

import static org.ow2.petals.binding.soap.Constants.Axis2.SERVICES_XML;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.ADDRESS;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.MODULES;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.POLICY_PATH;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.SERVICE_NAME;
import static org.ow2.petals.binding.soap.Constants.ServiceUnit.SERVICE_PARAMETERS;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.StringReader;
import java.net.URISyntaxException;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.deployment.DeploymentConstants;
import org.apache.axis2.deployment.DeploymentErrorMsgs;
import org.apache.axis2.deployment.DeploymentException;
import org.apache.axis2.deployment.ServiceBuilder;
import org.apache.axis2.description.AxisModule;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.i18n.Messages;
import org.apache.axis2.transport.jms.JMSConstants;
import org.apache.axis2.util.XMLUtils;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlFactory;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.petals.binding.soap.listener.incoming.PetalsAxisService;
import org.ow2.petals.binding.soap.listener.incoming.PetalsReceiver;
import org.ow2.petals.binding.soap.listener.incoming.SoapServerConfig;
import org.ow2.petals.binding.soap.listener.incoming.jetty.SoapServer;
import org.ow2.petals.binding.soap.util.SUPropertiesHelper;
import org.ow2.petals.binding.soap.util.SoapSUClassLoader;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.bc.AbstractBindingComponent;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.BindingComponentServiceUnitManager;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.util.ClassLoaderUtil;
import org.w3c.dom.Document;

/**
 * A service unit listener used to register new service into Axis Engine during
 * SU deployment.
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 */
public class SoapSUManager extends BindingComponentServiceUnitManager {

	/**
	 * The JBI component context
	 */
	private ComponentContext componentContext;

	/**
	 * The SOAP component context (configuration values)
	 */
	private SoapComponentContext soapContext;
	private SoapComponent soapComponent;

	/**
	 * JDK logger
	 */
	private Logger logger = null;

	/**
     * 
     */
	private SoapServerConfig soapServerConfig;

	private PetalsReceiver petalsReceiver;

	public SoapSUManager(final AbstractBindingComponent bindingComponent) {
		super(bindingComponent);
		soapComponent = (SoapComponent)bindingComponent;
	}

	/**
	 * Creates the OMElement corresponding to the parameters String, included in
	 * parameters tags.
	 * 
	 * @param parameters
	 *            the parameters
	 * @return Returns <code>OMElement</code> .
	 * @throws javax.xml.stream.XMLStreamException
	 */
	private final OMElement buildParametersOM(String parameters) throws XMLStreamException {
		OMElement element = null;
		if (parameters != null) {
			parameters = "<parameters>" + parameters + "</parameters>";
			element = (OMElement) XMLUtils.toOM(new StringReader(parameters));
			if (element != null) {
				element.build();
			}
		}
		return element;
	}

	/**
	 * Check and display warning for Consume parameters
	 * 
	 * @param extensions
	 */
	protected void checkConsume(final ConfigurationExtensions extensions) {
		// just check the service name parameter
		if (SUPropertiesHelper.getServiceName(extensions) == null) {
			// allow address
			if (SUPropertiesHelper.getAddress(extensions) == null) {
				logger.warning("The field " + SERVICE_NAME + " can't be found into the consumes");
			} else {
				logger.info("The field " + ADDRESS + " is deprecated, please use " + SERVICE_NAME);
			}
		}
	}

	/**
	 * Check and display warnings for Provide parameters
	 * 
	 * @param extensions
	 */
	protected void checkProvide(final ConfigurationExtensions extensions) {
		// just check the address field
		// allow address
		if (SUPropertiesHelper.getWSATo(extensions) == null && SUPropertiesHelper.getAddress(extensions) == null) {
			if (logger.isLoggable(Level.INFO)) {
				logger.info("The field 'wsa-to' isn't specified into the extensions. We'll search the WS-Addressing into the exchanges.");
			}
		}
	}

	/**
	 * Clean the context
	 * 
	 * @param serviceUnitName
	 * @param descriptor
	 */
	private void cleanServiceContexts(final String serviceUnitName, final Jbi descriptor) {
		soapContext.removeJbiDescriptor(serviceUnitName);
		soapContext.removeServiceDescriptor(serviceUnitName);

		final List<Provides> providesList = descriptor.getServices().getProvides();
		ServiceUnitDataHandler suDatahandler = null;

		for (final Provides provides : providesList) {
			if (suDatahandler == null) {
				suDatahandler = getSUDataHandlerForService(provides);
			}

			if (logger.isLoggable(Level.FINE)) {
				logger.fine("Cleaning context for " + provides);
			}
			soapContext.getProvidersManager().deleteServiceContext(provides);
			soapContext.deleteServiceClientPools(provides);
		}

		final List<Consumes> consumesList = descriptor.getServices().getConsumes();
		for (final Consumes consumes : consumesList) {
			if (suDatahandler == null) {
				suDatahandler = getSUDataHandlerForService(consumes);
			}

			if (logger.isLoggable(Level.FINE)) {
				logger.fine("Cleaning context for " + consumes);
			}
			soapContext.getConsumersManager().deleteServiceContext(consumes);
		}
	}

	/**
	 * Create the Axis service that will handle incoming SOAP calls whatever the
	 * transport layer.
	 * 
	 * @param consumes
	 * @param serviceDescriptor
	 * @throws PEtALSComponentSDKException
	 */
	private void createAxisService(final Consumes consumes, final File serviceDescriptor, final ConfigurationExtensions extensions) throws PEtALSCDKException {
		final String localAddress = soapServerConfig.getServicesURL();
		// Get the created service name
		String newServiceName = SUPropertiesHelper.getServiceName(extensions);
		// allow address
		if (newServiceName == null) {
			newServiceName = SUPropertiesHelper.getAddress(extensions);
			if (logger.isLoggable(Level.FINE)) {
				logger.fine("We're using the deprecated address attribute. Service key: " + newServiceName + ", service address: " + localAddress);
			}
		} else if (logger.isLoggable(Level.FINE)) {
			logger.fine("We're using the service name attribute. Service key: " + newServiceName + ", service address: " + localAddress);
		}
		try {
			// try to find if the axisService has already been registered
			final AxisConfiguration axisConfig = soapContext.getAxis2ConfigurationContext().getAxisConfiguration();
			if (axisConfig.getService(newServiceName) != null) {
				// The Axis service already exists
				logger.log(Level.WARNING, "The service '" + newServiceName + "' is already registered in Axis, you cannot register it again");
			} else {
				// The Axis service does not exist, we create it.
				final AxisService axisService = new PetalsAxisService(newServiceName, componentContext, consumes, extensions, localAddress, petalsReceiver,
						logger);
				
                // set the transport layers 
				// (necessary to set the transport before adding the Axis service to Axis configuration)
                axisService.setEnableAllTransports(false);
                setTransportHttpToAxisService(axisService, extensions);
                setTransportJmsToAxisService(axisService, extensions);
				
				// populate service with service descriptor
				final QName jbiServiceQName = consumes.getServiceName();
				if (jbiServiceQName != null) {
					axisService.setTargetNamespace(jbiServiceQName.getNamespaceURI());
				}
				axisService.addParameter(new Parameter(Constants.SERVICE_CLASS, "PetalsReceiver"));
				axisService.addParameter(new Parameter(Constants.Configuration.SEND_STACKTRACE_DETAILS_WITH_FAULTS, Boolean.TRUE));
				
				setServiceParametersToAxisService(axisService, consumes, serviceDescriptor);
				setModulesToAxisService(axisService, consumes);
			}
		} catch (final AxisFault e) {
			throw new PEtALSCDKException("Can not register Service into Axis context", e);
		}
	}

	/**
	 * Create the services context
	 * 
	 * @param serviceUnitName
	 * @param suRootPath
	 * @param descriptor
	 */
	private void createServicesContext(final String serviceUnitName, final String suRootPath, final Jbi descriptor) {
		ServiceUnitDataHandler suDatahandler = null;

		final List<Provides> providesList = descriptor.getServices().getProvides();
		for (final Provides provides : providesList) {
			if (suDatahandler == null) {
				suDatahandler = getSUDataHandlerForService(provides);
			}

			final ServiceContext<Provides> context = soapContext.getProvidersManager().createServiceContext(provides);
			final ConfigurationExtensions extensions = suDatahandler.getConfigurationExtensions(provides);
			context.setModules(getModules(extensions));
			context.setServiceParams(getServiceParameters(extensions));
			context.setPolicyPath(getPolicyPath(suRootPath, extensions));
			context.setClassloader(getServiceClassloader(suRootPath, extensions));
		}

		final List<Consumes> consumesList = descriptor.getServices().getConsumes();
		for (final Consumes consumes : consumesList) {
			if (suDatahandler == null) {
				suDatahandler = getSUDataHandlerForService(consumes);
			}

			final ServiceContext<Consumes> context = soapContext.getConsumersManager().createServiceContext(consumes);
			final ConfigurationExtensions extensions = suDatahandler.getConfigurationExtensions(consumes);
			context.setModules(getModules(extensions));
			context.setServiceParams(getServiceParameters(extensions));
			context.setPolicyPath(getPolicyPath(suRootPath, extensions));
			context.setClassloader(getServiceClassloader(suRootPath, extensions));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.ow2.petals.component.framework.su.AbstractServiceUnitManager#doDeploy
	 * (java.lang.String, java.lang.String,
	 * org.ow2.petals.component.framework.jbidescriptor.generated.Jbi)
	 */
	@Override
	protected void doDeploy(final String serviceUnitName, final String suRootPath, final Jbi jbiDescriptor) throws PEtALSCDKException {
		logger.log(Level.FINE, "Deploying a new AxisService for SU " + serviceUnitName);

		// Global service unit configuration
		soapContext.addJbiDescriptor(serviceUnitName, jbiDescriptor);

		final File servicesXml = new File(suRootPath, SERVICES_XML);
		if (servicesXml.exists()) {
			soapContext.addServiceDescriptor(serviceUnitName, servicesXml);
		}
		// Local service configuration
		createServicesContext(serviceUnitName, suRootPath, jbiDescriptor);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.ow2.petals.component.framework.su.AbstractServiceUnitManager#doStart
	 * (java.lang.String)
	 */
	@Override
	protected void doStart(final String serviceUnitName) throws PEtALSCDKException {
		// get descriptors
		final Jbi descriptor = soapContext.getJbiDescriptor(serviceUnitName);
		final File serviceDescription = soapContext.getServiceDescriptor(serviceUnitName);

		if (descriptor != null) {

			ServiceUnitDataHandler suDatahandler = null;
			/*
			 * Register new Axis2 service only on consumes nodes. Create an
			 * AxisService for each requested endpoint. If an enpoint has not
			 * been found, simply log a severe error. TODO : If a service
			 * creation fails, and if services have been already registered for
			 * this service unit, unregister them.
			 */
			final List<Consumes> consumesList = descriptor.getServices().getConsumes();
			for (final Consumes consumes : consumesList) {
				if (suDatahandler == null) {
					suDatahandler = getSUDataHandlerForService(consumes);
				}
				final ConfigurationExtensions extensions = suDatahandler.getConfigurationExtensions(consumes);
				checkConsume(extensions);
				createAxisService(consumes, serviceDescription, extensions);

				String redirect = SUPropertiesHelper.getHttpRedirection(extensions);
				if(redirect != null) {
					SoapServer httpServer = soapComponent.getExternalListenerManager().getHttpServer();
					String serviceName = SUPropertiesHelper.getServiceName(extensions);
					if(serviceName == null) {
						serviceName = SUPropertiesHelper.getAddress(extensions);
					}
					// Comma-separated list of redirection aliases
					StringTokenizer st = new StringTokenizer(redirect, ",;|");
					while(st.hasMoreTokens()) {
						httpServer.addRedirect(st.nextToken().trim(), serviceName);
					}
				}
			}
			// create/restore services
			final List<Provides> providesList = descriptor.getServices().getProvides();
			for (final Provides provides : providesList) {
				if (suDatahandler == null) {
					suDatahandler = getSUDataHandlerForService(provides);
				}
				final ConfigurationExtensions extensions = suDatahandler.getConfigurationExtensions(provides);
				checkProvide(extensions);
				final ServiceContext<Provides> context = soapContext.getProvidersManager().getServiceContext(provides);
				final ServiceEndpoint srvEp = componentContext.getEndpoint(provides.getServiceName(), provides.getEndpointName());
				Document doc;
				try {
					doc = componentContext.getEndpointDescriptor(srvEp);
					context.setServiceDescription(WSDL4ComplexWsdlFactory.newInstance().newWSDLReader().read(doc));
				} catch (final JBIException e) {
					logger.warning("No endpoint descriptor found for the service endpoint : " + srvEp);
				} catch (final WSDL4ComplexWsdlException e) {
					logger.warning("Wsdl reading error" + e.getMessage());
				} catch (final URISyntaxException e) {
					logger.warning("Wsdl reading error" + e.getMessage());
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.ow2.petals.component.framework.su.AbstractServiceUnitManager#doStop
	 * (java.lang.String)
	 */
	@Override
	protected void doStop(final String serviceUnitName) throws PEtALSCDKException {

		final Jbi descriptor = soapContext.getJbiDescriptor(serviceUnitName);
		if (descriptor != null) {
			ServiceUnitDataHandler suDatahandler = null;

			// delete registered axis services
			final List<Consumes> consumesList = descriptor.getServices().getConsumes();
			for (final Consumes consumes : consumesList) {
				if (suDatahandler == null) {
					suDatahandler = getSUDataHandlerForService(consumes);
				}
				final ConfigurationExtensions extensions = suDatahandler.getConfigurationExtensions(consumes);
				unregisterAxisService(extensions);
				
				String redirect = extensions.get("http-services-redirection");
				if(redirect != null) {
					// Comma-separated list of redirection aliases
					StringTokenizer st = new StringTokenizer(redirect, ",;|");
					while(st.hasMoreTokens()) {
						soapComponent.getExternalListenerManager().getHttpServer().removeRedirect(st.nextToken().trim());
					}
				}
			}
		}
	}

	@Override
	protected void doUndeploy(final String serviceUnitName) throws PEtALSCDKException {
		final Jbi descriptor = getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
		cleanServiceContexts(serviceUnitName, descriptor);
	}

	/**
	 * Get the modules
	 * 
	 * @param provides
	 */
	private List<String> getModules(final ConfigurationExtensions extensions) {
		final List<String> result = new ArrayList<String>();

		// get modules from extension
		final String token = extensions.get(MODULES);

		if (token != null) {
			// get individual modules values
			final StringTokenizer st = new StringTokenizer(token, ",");
			result.add(Constants.MODULE_ADDRESSING);
			while (st.hasMoreTokens()) {
				result.add(st.nextToken().trim());
			}
		}
		return result;
	}

	/**
	 * Process the parameterElement object from the OM, and returns the
	 * corresponding Parameter.
	 * 
	 * @param parameterElement
	 *            <code>OMElement</code>
	 * @return the Parameter parsed
	 * @throws DeploymentException
	 *             if bad paramName
	 */
	private Parameter getParameter(final OMElement parameterElement) throws DeploymentException {

		final Parameter parameter = new Parameter();
		// setting parameterElement
		parameter.setParameterElement(parameterElement);
		// setting parameter Name
		final OMAttribute paramName = parameterElement.getAttribute(new QName(DeploymentConstants.ATTRIBUTE_NAME));
		if (paramName == null) {
			throw new DeploymentException(Messages.getMessage(DeploymentErrorMsgs.BAD_PARAMETER_ARGUMENT, parameterElement.toString()));
		}
		parameter.setName(paramName.getAttributeValue());
		// setting parameter Value (the child element of the parameter)
		final OMElement paramValue = parameterElement.getFirstElement();
		if (paramValue != null) {
			parameter.setValue(paramValue);
			parameter.setParameterType(Parameter.OM_PARAMETER);
		} else {
			final String paratextValue = parameterElement.getText();

			parameter.setValue(paratextValue);
			parameter.setParameterType(Parameter.TEXT_PARAMETER);
		}

		return parameter;
	}

	/**
	 * Get the the policy path
	 * 
	 * @param suRootPath
	 * @param extensions
	 */
	private File getPolicyPath(final String suRootPath, final ConfigurationExtensions extensions) {
		final String policyPath = extensions.get(POLICY_PATH);
		File result = null;

		if (policyPath == null) {
			return result;
		}

		final File path = new File(suRootPath, policyPath);
		if (logger.isLoggable(Level.FINE)) {
			logger.fine("Setting policy path to " + path.getAbsolutePath());
		}

		if (path.exists() && path.isDirectory()) {
			result = path;
		} else {
			logger.warning("The policy path does not exists, it will be ignored " + path.getAbsolutePath());
		}
		return result;
	}

	/**
	 * Get the service class loader
	 * 
	 * @param suRootPath
	 * @param extensions
	 */
	private URLClassLoader getServiceClassloader(final String suRootPath, final ConfigurationExtensions extensions) {
		return ClassLoaderUtil.createClassLoader(suRootPath, Thread.currentThread().getContextClassLoader());
	}

	/**
	 * Add parameters for the associated service to the componentContext.
	 * 
	 * @param extensions
	 */
	private String getServiceParameters(final ConfigurationExtensions extensions) {
		return extensions.get(SERVICE_PARAMETERS);
	}

	/**
	 * Creates a new instance of {@link SoapSUManager}
	 * 
	 * @param soapContext
	 * @param axisConfigurationContext
	 * @param componentContext
	 * @param servicesURL
	 * @param logger
	 */
	public void init(final SoapComponentContext soapContext, final ComponentContext componentContext, final SoapServerConfig soapServerConfig,
			final PetalsReceiver petalsReceiver, final Logger logger) {
		this.soapContext = soapContext;
		this.componentContext = componentContext;
		this.soapServerConfig = soapServerConfig;
		this.petalsReceiver = petalsReceiver;
		this.logger = logger;
	}

	/**
	 * Set and engage needed module to an Axis service
	 * 
	 * @param axisService
	 * @param consumes
	 * @throws DeploymentException
	 * @throws AxisFault
	 */
	private void setModulesToAxisService(final AxisService axisService, final Consumes consumes) throws DeploymentException, AxisFault {
		// We prepare a list for the modules ClassLoaders
		List<ClassLoader> suModulesClassLoaders = null;

		/*
		 * Add the modules for this new service. If the services.xml file if
		 * available, the modules references can be redefined
		 */
		final List<String> modules = soapContext.getConsumersManager().getModules(consumes);
		if (modules != null) {
			suModulesClassLoaders = new ArrayList<ClassLoader>();
			final AxisConfiguration axisConfig = soapContext.getAxis2ConfigurationContext().getAxisConfiguration();
			for (final String module : modules) {
				final AxisModule axisModule = axisConfig.getModule(module);
				if (axisModule == null) {
					throw new DeploymentException(Messages.getMessage(DeploymentErrorMsgs.MODULE_NOT_FOUND, module));
				}
				axisService.addModuleref(axisModule.getName());

				// We add the current module ClassLoader to the list
				suModulesClassLoaders.add(axisModule.getModuleClassLoader());
			}

			final String suRootPath = getSUDataHandlerForConsumes(consumes).getInstallRoot();

			// We set the service ClassLoader = initial ClassLoader
			// + jars of suRootPath + modules ClassLoaders
			axisService.setClassLoader(new SoapSUClassLoader(ClassLoaderUtil.getUrls(suRootPath), suModulesClassLoaders, axisService.getClassLoader()));

			// Add the service before to engaging it otherwise a NPE occur
			soapContext.getAxis2ConfigurationContext().getAxisConfiguration().addService(axisService);

			// engage the modules if needed
			for (final String module : modules) {
				final AxisModule axisModule = axisConfig.getModule(module);
				if (!axisService.isEngaged(axisModule)) {
					axisService.engageModule(axisModule, axisService);
				}
			}
		}
	}

	/**
	 * Set to an Axis service the service parameters provided as CDATA from the
	 * extension 'service-parameters' or in the embedded file 'services.xml'.
	 * 
	 * @param axisService
	 * @param consumes
	 * @throws AxisFault
	 */
	@SuppressWarnings("unchecked")
	private void setServiceParametersToAxisService(final AxisService axisService, final Consumes consumes, final File serviceDescriptor) throws AxisFault {
		// we set the associated service parameters
		final String serviceParameters = soapContext.getConsumersManager().getServiceParameters(consumes);
		if (serviceParameters != null) {
			try {
				final OMElement parametersElements = buildParametersOM(serviceParameters);

				// get an iterator on all <parameter> children
				final Iterator<OMElement> itr = parametersElements.getChildrenWithName(new QName(DeploymentConstants.TAG_PARAMETER));

				// iterate on parameters and set them to the associated
				// axisService
				while (itr.hasNext()) {
					final OMElement parameterElement = itr.next();

					if (DeploymentConstants.TAG_PARAMETER.equalsIgnoreCase(parameterElement.getLocalName())) {
						axisService.addParameter(getParameter(parameterElement));
					}
				}
			} catch (final XMLStreamException e) {
				logger.warning(e.getMessage());
			} catch (final DeploymentException e) {
				logger.warning(e.getMessage());
			}
		}

		// Try to load axis configuration from services.xml
		if (serviceDescriptor != null) {
			try {
				final FileInputStream fis = new FileInputStream(serviceDescriptor);
				final ServiceBuilder builder = new ServiceBuilder(fis, soapContext.getAxis2ConfigurationContext(), axisService);
				builder.populateService(builder.buildOM());
			} catch (final XMLStreamException e) {
				logger.warning(e.getMessage());
			} catch (final FileNotFoundException e) {
				logger.warning(e.getMessage());
			} catch (final DeploymentException e) {
				logger.warning(e.getMessage());
			}
		}
	}

	/**
	 * Set an Axis service as a SOAP over HTTP service.
	 * 
	 * @param axisService
	 */
	private void setTransportHttpToAxisService(final AxisService axisService, final ConfigurationExtensions extensions) {

		if (SUPropertiesHelper.isHttpTransportEnable(extensions)) {
			axisService.addExposedTransport(Constants.TRANSPORT_HTTP);

			logger.log(Level.INFO, "The Axis2 service '" + axisService.getName() + "' has been registered and is available at '"
					+ soapServerConfig.getServicesURL() + "/" + axisService.getName() + "'");
		}
	}

	/**
	 * Set an Axis service as a SOAP over JMS service.
	 * 
	 * @param axisService
	 * @throws AxisFault
	 */
	private void setTransportJmsToAxisService(final AxisService axisService, final ConfigurationExtensions extensions) throws AxisFault {

		if (SUPropertiesHelper.isJmsTransportEnable(extensions)) {

			axisService.addParameter(new Parameter(JMSConstants.DEST_PARAM, axisService.getName()));

			axisService.addExposedTransport(Constants.TRANSPORT_JMS);

			logger.log(Level.INFO, "The Axis2 service '" + axisService.getName() + "' has been registered and is available through JMS.");
		}
	}

	/**
	 * Unregister the service from Axis.
	 * 
	 * @param serviceQName
	 * @throws PEtALSComponentSDKException
	 */
	private void unregisterAxisService(final ConfigurationExtensions extensions) throws PEtALSCDKException {
		String serviceName = SUPropertiesHelper.getServiceName(extensions);
		if (serviceName == null) {
			serviceName = SUPropertiesHelper.getAddress(extensions);
		}
		final AxisConfiguration axisConfig = soapContext.getAxis2ConfigurationContext().getAxisConfiguration();
		logger.log(Level.INFO, "Removing Axis service '" + serviceName + "'");
		try {
			// register an axis service to axis engine
			final AxisService axisService = axisConfig.getService(serviceName);
			// Feature request : #306664
			// FIXME : We do not have to remove the service group. It is
			// temporary until the next version of Axis2 where the removeService
			// method will fix it.
			if (axisService != null) {
				axisConfig.removeServiceGroup(serviceName);
				axisService.getAxisConfiguration().removeService(serviceName);
			} else {
				logger.log(Level.WARNING, "Service '" + serviceName + "' not found, can not be unregistered from Axis2");
			}
		} catch (final AxisFault e) {
			throw new PEtALSCDKException("Can not remove service from Axis context", e);
		}
	}
}
