/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap.util;

import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlFactory;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlWriter;
import org.ow2.easywsdl.wsdl.api.Binding;
import org.ow2.easywsdl.wsdl.api.BindingOperation;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfBinding.StyleConstant;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 * @author "Mathieu CARROLLE - eBM WebSourcing"
 */
public class WsdlHelper {

	/**
	 * Creates a new instance of {@link WsdlHelper}
	 */
	private WsdlHelper() {
	}

	/**
	 * Replace the endpoint address in the WSDL
	 * 
	 * @return
	 * @throws WSDL4ComplexWsdlException
	 * @throws URISyntaxException
	 */
	public static final Document replaceServiceAddressInWSDL(final Document doc, final String address) throws WSDL4ComplexWsdlException, URISyntaxException {
		Document result = null;
		final WSDL4ComplexWsdlFactory wsdlFactory = WSDL4ComplexWsdlFactory.newInstance();
		final WSDL4ComplexWsdlWriter wsdlWriter = wsdlFactory.newWSDLWriter();
		final WSDL4ComplexWsdlReader reader = wsdlFactory.newWSDLReader();
		final Description desc = reader.read(doc);
		final List < Service > services = desc.getServices();
		for (final Service service : services) {
			final List < Endpoint > endpoints = service.getEndpoints();
			for (final Endpoint endpoint : endpoints) {
				endpoint.setAddress(address);
			}
		}
		result = wsdlWriter.getDocument(desc);
		return result;
	}

	/***
	 * Read soap action in the wsdl and store for the given service and endpoint name
	 * 
	 * @param doc
	 *            associated wsdl
	 * @param serviceName
	 * @param endpointName
	 * @return a map where the keys is a bindingOperation name and the value is the associated soap action
	 * @throws URISyntaxException
	 * @throws WSDL4ComplexWsdlException
	 */
	public static final Map < String, String > readAndStoreSoapAction(final Document doc, final QName serviceName, final String endpointName)
			throws WSDL4ComplexWsdlException, URISyntaxException {
		// Map of BindingOperation/soap action
		final Map < String, String > map = new HashMap < String, String >();
		final Description desc = WSDL4ComplexWsdlFactory.newInstance().newWSDLReader().read(doc);
		final Service srv = desc.getService(serviceName);
		final Endpoint ep = srv.getEndpoint(endpointName);
		final Binding bd = ep.getBinding();
		if (bd == null) {
			return map;
		}
		final List < BindingOperation > bOperations = bd.getBindingOperations();
		for (final BindingOperation op : bOperations) {
			map.put(op.getQName().getLocalPart(), op.getSoapAction());
		}
		return map;
	}

	public static BindingOperation findOperationUsingElement(final Element firstElement, final Description desc, final String endpointName,
			final QName serviceName) {
		BindingOperation result = null, guess = null;

		// Trying to retrieve the Binding
		final Binding b = desc.getService(serviceName).getEndpoint(endpointName).getBinding();

		if (firstElement != null) {
			for (final BindingOperation op : b.getBindingOperations()) {
				if (op.getStyle().equals(StyleConstant.RPC)) {
					if (op.getOperation().getQName().getNamespaceURI().equals(firstElement.getNamespaceURI())
							&& op.getOperation().getQName().getLocalPart().equals(firstElement.getLocalName())) {
						result = op;
						break;
					}
				} else {
					final QName elmtName = new QName(firstElement.getNamespaceURI(), org.ow2.easywsdl.wsdl.util.Util.getLocalPartWithoutPrefix(firstElement
							.getNodeName()));

					final org.ow2.easywsdl.schema.api.Element e = op.getOperation().getInput().getElement();
					if (e != null && e.getQName() != null && e.getQName().getLocalPart().equals(elmtName.getLocalPart())) {

						guess = op;
						if (e.getQName().getNamespaceURI().equals(elmtName.getNamespaceURI())) {
							result = op;
							break;
						}
					}
				}
			}
		}
		if (result == null) {
			result = guess; // Best choice possible... URI not matched, but localpart OK
		}
		return result;
	}

	public static BindingOperation findOperationUsingSoapAction(final String soapAction, final Description desc) {
		BindingOperation res = null;
		boolean find = false;
		for (final Binding b : desc.getBindings()) {
			for (final BindingOperation op : b.getBindingOperations()) {
				if (op.getSoapAction() != null && op.getSoapAction().equals(soapAction)) {
					res = op;
					find = true;
					break;
				}
			}
			if (find) {
				break;
			}
		}
		return res;
	}

	/***
	 * Retrieve soapAction in the WSDL depending on the first element of the request
	 * 
	 * @param firstElement
	 *            the first element of the request
	 * @param desc
	 *            the WSDL of the service
	 * @return the soapAction of the operation
	 */
	public static String findSoapAction(final Element firstElement, final Description desc, final String endpointName, final QName serviceName) {
		final BindingOperation op = findOperationUsingElement(firstElement, desc, endpointName, serviceName);
		if (op != null) {
			return op.getSoapAction();
		} else {
			return "";
		}
	}

}
